<?php

use CKPL\Pay\Definition\Amount\AmountInterface;
use CKPL\Pay\Definition\Payment\Address\PaymentAddressInterface;
use CKPL\Pay\Definition\Payment\Builder\AmountBuilder;
use CKPL\Pay\Definition\Payment\Builder\AmountBuilderInterface;
use CKPL\Pay\Definition\Payment\Builder\PaymentAddressBuilder;
use CKPL\Pay\Definition\Payment\Builder\PaymentBuilder;
use CKPL\Pay\Definition\Payment\Builder\PaymentCustomerBuilder;
use CKPL\Pay\Definition\Payment\Builder\PaymentOrderBuilder;
use CKPL\Pay\Definition\Payment\Builder\PaymentOrderBuilderInterface;
use CKPL\Pay\Definition\Payment\Customer\PaymentCustomerInterface;
use CKPL\Pay\Definition\Payment\Order\PaymentOrderInterface;
use CKPL\Pay\Definition\Payment\PaymentInterface;
use CKPL\Pay\Exception\Definition\AmountException;
use CKPL\Pay\Exception\Definition\PaymentAddressException;
use CKPL\Pay\Exception\Definition\PaymentCustomerException;
use CKPL\Pay\Exception\Definition\PaymentException;

/**
 * Class WC_Gateway_Conotoxia_Pay_Utils
 */
class WC_Gateway_Conotoxia_Pay_Utils
{
    /**
     * @param WC_Order $order
     *
     * @param WC_Gateway_Conotoxia_Pay $gateway
     *
     * @return PaymentInterface
     * @throws AmountException
     * @throws PaymentAddressException
     * @throws PaymentCustomerException
     * @throws PaymentException
     */
    public static function createPaymentFromOrder(WC_Order $order, WC_Gateway_Conotoxia_Pay $gateway): PaymentInterface
    {
        $returnUrl = $gateway->get_return_url($order);
        $amount = static::createAmountFromOrder($order);
        $paymentOrder = static::createPaymentOrderFromWooCommerceOrder($order);

        $paymentBuilder = (new PaymentBuilder())
            ->setExternalPaymentId($order->get_id())
            ->setAmount($amount)
            ->setDescription(get_bloginfo('name') . ' '.__('order').' #' . $order->get_order_number())
            ->setReturnUrl($returnUrl)
            ->setErrorUrl($returnUrl)
            ->setNotificationUrl(add_query_arg('wc-api', 'WC_Gateway_Conotoxia_Pay', add_query_arg('type', 'payment', home_url('/'))));

        if ($paymentOrder !== null) {
            $paymentBuilder->setOrder($paymentOrder);
        }

        return $paymentBuilder->getPayment();
    }

    /**
     * @param WC_Order $order
     *
     * @return PaymentOrderInterface
     * @throws PaymentAddressException
     * @throws PaymentCustomerException
     * @throws PaymentException
     */
    public static function createPaymentOrderFromWooCommerceOrder(WC_Order $order): PaymentOrderInterface
    {
        $orderBuilder = new PaymentOrderBuilder();

        $orderBuilder
            ->setCustomer(static::createCustomerFromOrder($order))
            ->setAddress(static::createShippingAddressFromOrder($order));

        static::processProductFromWooCommerceOrderToPaymentOrder($order, $orderBuilder);


        return $orderBuilder->getPaymentOrder();
    }

    /**
     * @param WC_Order $order
     *
     * @return PaymentAddressInterface
     * @throws PaymentAddressException
     */
    public static function createShippingAddressFromOrder(WC_Order $order): PaymentAddressInterface
    {
        $shippingStreet = \join(' ', [$order->get_shipping_address_1(), $order->get_shipping_address_2()]);

        return (new PaymentAddressBuilder())
            ->setStreet($shippingStreet)
            ->setCity($order->get_shipping_city())
            ->setPostalCode($order->get_shipping_postcode())
            ->setCountry($order->get_shipping_country())
        ->getPaymentAddress();
    }

    /**
     * @param WC_Order $order
     *
     * @return PaymentAddressInterface
     * @throws PaymentAddressException
     */
    public static function createBillingAddressFromOrder(WC_Order $order): PaymentAddressInterface
    {
        $shippingStreet = \join(' ', [$order->get_billing_address_1(), $order->get_billing_address_2()]);

        return (new PaymentAddressBuilder())
            ->setStreet($shippingStreet)
            ->setCity($order->get_billing_city())
            ->setPostalCode($order->get_billing_postcode())
            ->setCountry($order->get_billing_country())
        ->getPaymentAddress();
    }

    /**
     * @param WC_Order $order
     *
     * @return PaymentCustomerInterface
     * @throws PaymentAddressException
     * @throws PaymentCustomerException
     */
    public static function createCustomerFromOrder(WC_Order $order): PaymentCustomerInterface
    {
        return (new PaymentCustomerBuilder())
            ->setFirstName($order->get_billing_first_name())
            ->setLastName($order->get_billing_last_name())
            ->setEmail($order->get_billing_email())
            ->setPhone($order->get_billing_phone())
            ->setAddress(static::createBillingAddressFromOrder($order))
        ->getPaymentCustomer();
    }

    /**
     * @param WC_Order $order
     *
     * @param PaymentOrderBuilderInterface $orderBuilder
     *
     * @return void
     * @throws PaymentException
     */
    public static function processProductFromWooCommerceOrderToPaymentOrder(WC_Order $order, PaymentOrderBuilderInterface $orderBuilder): void
    {
        foreach ($order->get_items() as $product) {
            if (!$product->is_type('line_item') || !($product instanceof WC_Order_Item_Product)) {
                continue;
            }

            $productSimple = wc_get_product($product->get_product());

            $orderBuilder->addProduct(
                $orderBuilder->createProductBuilder()
                     ->setName($product->get_name())
                     ->setQuantity($product->get_quantity())
                     ->buildAmount(function (AmountBuilderInterface $amountBuilder) use ($productSimple) {
                         $amountBuilder->setCurrency(get_woocommerce_currency());
                         $amountBuilder->setValue($productSimple->get_price());
                     })
                 ->getPaymentProduct()
            );
        }
    }

    /**
     * @param WC_Order $order
     *
     * @return AmountInterface
     * @throws AmountException
     */
    public static function createAmountFromOrder(WC_Order $order): AmountInterface
    {
        return (new AmountBuilder())
           ->setValue(\number_format($order->get_total(), 2, '.', ''))
           ->setCurrency(get_woocommerce_currency())
        ->getAmount();
    }
}
