<?php

use CKPL\Pay\Exception\Exception as PayException;
use CKPL\Pay\Pay;
use CKPL\Pay\Configuration\ConfigurationInterface;
use CKPL\Pay\Configuration\Factory\ConfigurationFactory;
use CKPL\Pay\Definition\Payment\Builder\AmountBuilderInterface;
use CKPL\Pay\Definition\Payment\Builder\PaymentOrderBuilderInterface;
use CKPL\Pay\Exception\ConfigurationException;
use CKPL\Pay\Exception\Definition\PaymentException;
use CKPL\Pay\Exception\Exception;
use CKPL\Pay\Exception\MerchantManagerException;

/**
 * Class WC_Gateway_ConotoxiaPay
 */
class WC_Gateway_Conotoxia_Pay extends WC_Payment_Gateway
{
    /**
     * @var Pay $conotoxiaPay
     */
    protected $conotoxiaPay;

    /**
     * @type string
     */
    const PAYMENTS_HOST = 'https://login.cinkciarz.pl/';

    /**
     * @type string
     */
    const OIDC_HOST = 'https://partner.cinkciarz.pl/';

    /**
     * @type string
     */
    const PAYMENTS_HOST_SANDBOX = 'https://pay-api.ckpl.us/';

    /**
     * @type string
     */
    const OIDC_HOST_SANDBOX = 'https://oidc.ckpl.us/';

    /**
     * WC_Gateway_Conotoxia_Pay constructor.
     * @throws ConfigurationException
     */
    public function __construct()
    {
        $this->id = 'conotoxia_pay';
        $this->title = __('Conotoxia Pay', 'conotoxia-pay');
        $this->description = __('Pay using your Conotoxia account.');
        $this->has_fields = false;
        $this->icon = apply_filters('woocommerce_conotoxia-pay_icon', get_site_url() . '/wp-content/plugins/conotoxia-pay-woocommerce/images/pay_logo.svg');
        $this->method_title = __('Conotoxia Pay', 'Conotoxia-pay');
        $this->method_description = __('Official Conotoxia Pay payment gateway for WooCommerce.', 'conotoxia-pay');
        $this->supports = [
            'products',
            'refunds'
        ];

        $this->init_settings();
        $this->init_form_fields();

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
        add_action('woocommerce_api_wc_gateway_' . $this->id, [$this, 'processIncomingRequest']);
	    add_action('admin_notices', [$this, 'processAdminOptionsError']);

	    add_filter('woocommerce_thankyou_order_received_text', [$this, 'processOrderReceived'], 10, 2 );

        $this->initializeConotoxiaPay();
        $this->generateRsaKeys();
    }

    /**
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = [
            'enabled' => [
                'title' => __('Enable/Disable', 'woocommerce'),
                'label' => __('Enable Conotoxia Pay payment method', 'conotoxia-pay'),
                'type' => 'checkbox',
                'default' => 'no',
            ],
            'sandbox' => [
                'title' => __('Sandbox mode', 'conotoxia-pay'),
                'label' => __('Enable sandbox mode to test payments.', 'conotoxia-pay'),
                'type' => 'checkbox',
                'default' => 'no',
            ],
            'client_id' => [
                'title' => __('Client ID', 'conotoxia-pay'),
                'type' => 'text',
                'description' => __('Client ID is generated with Point of Sale ID.', 'conotoxia-pay'),
                'desc_tip' => true
            ],
            'client_secret' => [
                'title' => __('Client secret', 'conotoxia-pay'),
                'type' => 'text',
                'description' => __('Client secret is generated with Point of Sale ID.', 'conotoxia-pay'),
                'desc_tip' => true
            ],
            'point_of_sale' => [
                'title' => __('Point of Sale ID', 'conotoxia-pay'),
                'type' => 'text',
                'description' => __('Point of Sale ID can be generated in merchant panel.', 'conotoxia-pay'),
                'desc_tip' => true
            ],
            'public_key' => [
                'title' => __('RSA public key', 'conotoxia-pay'),
                'type' => 'textarea',
                'description' => nl2br(
                    __('Generated RSA public key that will be sent to Conotoxia Pay.', 'conotoxia-pay').\PHP_EOL.
                    '<a href="?page=wc-settings&tab=checkout&section=conotoxia_pay&generate_keys">'.__('Generate public and private key', 'conotoxia-pay').'</a>'
                ),
                'help' => 'Test'
            ],
            'private_key' => [
                'title' => __('RSA private key', 'conotoxia-pay'),
                'type' => 'textarea',
                'description' => nl2br(
                    __('Generated RSA private key that will be used to sign request.', 'conotoxia-pay').\PHP_EOL.
                    '<a href="?page=wc-settings&tab=checkout&section=conotoxia_pay&generate_keys">'.__('Generate public and private key', 'conotoxia-pay').'</a>'
                ),
            ]
        ];
    }

    /**
     * @param int $orderId
     *
     * @return array|bool
     */
    public function process_payment($orderId)
    {
        $order = wc_get_order($orderId);

        try {
            $payment = WC_Gateway_Conotoxia_Pay_Utils::createPaymentFromOrder($order, $this);
            $createdPayment = $this->conotoxiaPay->payments()->makePayment($payment);

            wc_reduce_stock_levels($order->get_id());

            WC()->cart->empty_cart();

            $order->update_status('on-hold', __('Payments has been sent to Conotoxia Pay. Waiting for confirmation.', 'conotoxia-pay'));

            add_post_meta($orderId, '_transaction_id', $createdPayment->getPaymentId(), true);

            return [
                'result' => 'success',
                'redirect' => $createdPayment->getApproveUrl()
            ];
        } catch (Exception $e) {
            wc_add_notice(
                __('Payment error', 'conotoxia-pay') . ': ' . $e->getMessage() . ' (' . $e->getCode() . ')',
                'error'
            );

            return false;
        }
    }

    /**
     * @param int $orderId
     * @param null $amount
     * @param string $reason
     *
     * @return bool|WP_Error
     */
    public function process_refund($orderId, $amount = null, $reason = '')
    {
        $order = wc_get_order($orderId);
        $conotoxiaOrderId = $order->get_transaction_id();

        if (empty($conotoxiaOrderId)) {
            return false;
        }

        $refundBuilder = $this->conotoxiaPay->refunds()->createRefundBuilder();

        $refund = $refundBuilder->setReason(
            empty($reason)
                ? __('Refund of', 'conotoxia') . ': ' . $amount . $order->get_currency() . ' ' . __('for order', 'conotoxia-pay') . ': ' . $orderId
                : $reason
            )
            ->setValue($this->formatAmount($amount))
            ->setCurrency($order->get_currency())
            ->setExternalRefundId($orderId)
            ->setPaymentId($conotoxiaOrderId)
            ->setNotificationUrl(
                add_query_arg('wc-api', 'WC_Gateway_Conotoxia_Pay', add_query_arg('type', 'refund', home_url('/')))
            )
        ->getRefund();

        try {
            $createdRefund = $this->conotoxiaPay->refunds()->makeRefund($refund);

            $order->add_meta_data('_refund', ['id' => $createdRefund->getId(), 'amount' => $this->formatAmount($amount)]);
        } catch (\Exception $exception) {
            return new WP_Error('conotoxia-pay', $exception->getMessage());
        }

        return true;
    }

    /**
     * @return bool
     */
    public function process_admin_options(): bool
    {
        $processed = parent::process_admin_options();

        try {
            $this->initializeConotoxiaPay();
            $this->conotoxiaPay->merchant()->invalidateToken();

            if ($processed && $this->isConfigurationCompleted()) {
                $this->conotoxiaPay->pickSignatureKey();

                if (
                    isset($_REQUEST['woocommerce_conotoxia_pay_public_key'])
                    && !empty($_REQUEST['woocommerce_conotoxia_pay_public_key'])
                ) {
                    try {
                        $this->conotoxiaPay->merchant()->pickPublicKeyId();
                    } catch (MerchantManagerException $exception) {
                        $this->conotoxiaPay->merchant()->sendPublicKey();
                    }
                }
            }
        } catch (Exception $exception) {
            $_SESSION['conotoxia_pay_error'] = $exception->getMessage();

            $processed = false;
        }

        return $processed;
    }

    /**
     * @return void
     */
    public function processIncomingRequest()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $input = file_get_contents('php://input');

            if ($_REQUEST['type'] === 'payment') {
                $this->processIncomingPayment($input);
            } elseif ($_REQUEST['type'] === 'refund') {
                $this->processIncomingRefund($input);
            } else {
                $this->sendResponse('HTTP/1.1 400 Bad Request', 'ERROR');

                return;
            }
        }

        $this->sendResponse('HTTP/1.1 200 OK', 'OK');
    }

    /**
     * @return void
     */
    public function processAdminOptionsError(): void
    {
        if (isset($_SESSION['conotoxia_pay_error']) && !empty($_SESSION['conotoxia_pay_error'])) {
            echo '<div class="notice notice-error"><p><strong>'.__($_SESSION['conotoxia_pay_error'], 'conotoxia-pay') .'</strong></p></div>';

            unset($_SESSION['conotoxia_pay_error']);
        }
    }

	/**
	 * @return void
	 */
	public function processOrderReceived(): void
	{
		$data = $_REQUEST['data'] ?? null;

		$success = __('Payment has been sent successfully.', 'conotoxia-pay');
		$failed = __('Payment process failed.', 'conotoxia-pay');
		$unknown = __('Payment has been sent but our service didn\'t receive any information yet.', 'conotoxia-pay');

		if (empty($data)) {
			echo $unknown;
		} else {
			try {
				$decoded = $this->conotoxiaPay->payments()->decodeReturn( $data );

				switch ($decoded->getResult()) {
					case 'SUCCESS':
					case 'SUCCESS_WITH_PAY_LATER':
						echo $success;
						break;
					case 'REJECTED':
						echo $cancelled;
						break;
					default:
						echo $failed;
				}
			} catch ( PayException $e ) {
				echo $unknown;
			}
		}
	}

    /**
     * @param string $input
     *
     * @return void
     */
    protected function processIncomingPayment(string $input): void
    {
        $status = $this->conotoxiaPay->payments()->getPaymentStatus($input);

        $order_id = (int)$status->getExternalPaymentId();
        $transaction_id = $status->getPaymentId();

        $order = wc_get_order($order_id);

        if ($order->get_status() !== 'completed' && $order->get_status() !== 'processing') {
            if ($status->isCompleted()) {
                $order->payment_complete($transaction_id);
            } elseif ($status->isRejected()) {
                $order->update_status('cancelled', __('Payment has been cancelled.', 'conotoxia-pay'));
            } elseif ($status->isCancelled()) {
                $order->update_status('failed', __('Payment has been rejected.', 'conotoxia-pay'));
            }
        }
    }

    /**
     * @param string $input
     *
     * @return void
     */
    protected function processIncomingRefund(string $input): void
    {
        $status = $this->conotoxiaPay->refunds()->getRefundStatus($input);
        $orderId = (int)$status->getExternalRefundId();

        $order = wc_get_order($orderId);

        if ($status->isCompleted()) {
            $order->add_order_note(__('Refund').' '.$status->getRefundId().' '.__('is completed.'), 1);
        } elseif ($status->isProcessing()) {
            $order->add_order_note(__('Refund').' '.$status->getRefundId().' '.__('is in progress.'), 1);
        } elseif ($status->isNew()) {
            $order->add_order_note(__('Refund').' '.$status->getRefundId().' '.__('has been created.'), 1);
        } else {
            $order->add_order_note(__('Refund').' '.$status->getRefundId().' '.__('cannot be finished.'), 1);
        }

        $order->save();
    }

    /**
     * @param WC_Order $order
     * @param PaymentOrderBuilderInterface $orderBuilder
     *
     * @return void
     * @throws PaymentException
     */
    protected function processProductsToOrder(WC_Order $order, PaymentOrderBuilderInterface $orderBuilder): void
    {
        foreach ($order->get_items() as $product) {
            if (!$product->is_type('line_item') || !($product instanceof WC_Order_Item_Product)) {
                continue;
            }

            $productSimple = wc_get_product($product->get_product());

            $orderBuilder->addProduct(
                $orderBuilder->createProductBuilder()
                    ->setName($product->get_name())
                    ->setQuantity($product->get_quantity())
                    ->buildAmount(function (AmountBuilderInterface $amountBuilder) use ($productSimple) {
                        $amountBuilder->setCurrency(get_woocommerce_currency());
                        $amountBuilder->setValue($productSimple->get_price());
                    })
                ->getPaymentProduct()
            );
        }
    }

    /**
     * @param string $amount
     *
     * @return string
     */
    protected function formatAmount(string $amount): string
    {
        return \number_format($amount, 2, '.', '');
    }

    /**
     * @return void
     */
    protected function generateRsaKeys(): void
    {
        if (isset($_GET['generate_keys'])) {
            if (!\extension_loaded('openssl')) {
                $_SESSION['conotoxia_pay_error'] = 'Missing OpenSSL extension.';
            } else {
                $pkGenerate = openssl_pkey_new([
                    'private_key_bits' => 2048,
                    'private_key_type' => OPENSSL_KEYTYPE_RSA
                ]);

                openssl_pkey_export($pkGenerate, $pkGeneratePrivate);

                $pkGenerateDetails = openssl_pkey_get_details($pkGenerate);
                $pkGeneratePublic = $pkGenerateDetails['key'];

                openssl_pkey_free($pkGenerate);

                $pkImport = openssl_pkey_get_private($pkGeneratePrivate);
                openssl_pkey_free($pkImport);

                $this->update_option('private_key', $pkGeneratePrivate);
                $this->update_option('public_key', $pkGeneratePublic);

                try {
                    if ($this->isConfigurationCompleted()) {
                        $this->initializeConotoxiaPay();

                        $this->conotoxiaPay->pickSignatureKey();
                        $this->conotoxiaPay->merchant()->sendPublicKey();
                    }

                    header('Location: /wp-admin/admin.php?page=wc-settings&tab=checkout&section=conotoxia_pay');
                    exit;
                } catch (Exception $exception) {
                    $_SESSION['conotoxia_pay_error'] = $exception->getMessage();
                }
            }
        }
    }

    /**
     * @return ConfigurationInterface
     * @throws ConfigurationException
     */
    protected function makeConfiguration(): ConfigurationInterface
    {
        $hostPayment = $this->get_option('sandbox') == 'yes' ? static::PAYMENTS_HOST_SANDBOX : static::PAYMENTS_HOST;
        $hostOidc = $this->get_option('sandbox') == 'yes' ? static::OIDC_HOST_SANDBOX : static::OIDC_HOST;

        return ConfigurationFactory::fromArray([
            ConfigurationInterface::HOST => $hostPayment,
            ConfigurationInterface::OIDC => $hostOidc,
            ConfigurationInterface::CLIENT_ID => $this->get_option('client_id'),
            ConfigurationInterface::CLIENT_SECRET => $this->get_option('client_secret'),
            ConfigurationInterface::POINT_OF_SALE => $this->get_option('point_of_sale'),
            ConfigurationInterface::PRIVATE_KEY => $this->get_option('private_key'),
            ConfigurationInterface::PUBLIC_KEY => $this->get_option('public_key'),
            ConfigurationInterface::STORAGE => new WC_Gateway_Conotoxia_Pay_Storage(),
        ]);
    }

    /**
     * @return bool
     */
    protected function isConfigurationCompleted(): bool
    {
        $required = ['client_id', 'client_secret', 'point_of_sale', 'private_key', 'public_key'];
        $result = true;

        foreach ($required as $option) {
            if (empty($this->get_option($option))) {
                $result = false;

                break;
            }
        }

        return $result;
    }

    /**
     * @param string $header
     * @param string $message
     * @return void
     */
    protected function sendResponse(string $header, string $message): void
    {
        \header($header);
        echo $message;

        \ob_flush();
    }

    /**
     * @return void
     * @throws ConfigurationException
     */
    protected function initializeConotoxiaPay(): void
    {
        $this->conotoxiaPay = new Pay($this->makeConfiguration());
    }
}
